-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

-- Code by SineMatic(YT)
-- If you'll be using or modifying this code, please let me know

local M = {}

local debugging = false
local initialFocusRangeMax
local dof = scenetree.findObject("DOFPostEffect") 
local rayCastFlags = bit.bor(SOTTerrain, SOTWater, SOTStaticShape, SOTPlayer, SOTItem, SOTVehicle, SOTForest) 
local sphereColor = ColorF(0, 1, 0, 0.7)
local textColor = ColorF(0, 1, 0, 1)
local textBackgroundColor = ColorI(0, 0, 0, 200)

local function round(num, numDecimalPlaces)
  local mult = 10^(numDecimalPlaces or 0)
  return math.floor(num * mult + 0.5) / mult
end

local function onUpdate()
	local rayCast = cameraMouseRayCast(true, rayCastFlags) or {} 
	dof.focalDist = rayCast.distance 
  
  if debugging and rayCast.object then
    local objectName = tostring(rayCast.object)
    local distance = round(rayCast.distance, 3)
    debugDrawer:drawTextAdvanced(rayCast.pos, ""..objectName.." Distance: "..distance .." m Focus value: "..(distance*2).."", textColor, true, false, textBackgroundColor)
    debugDrawer:drawSphere(rayCast.pos, 0.25, sphereColor)
  end
end

local function toggleManualFocus(value, debug)
  if value == 1 then
    extensions.load(M)
    print('manual focus enabled')
    initialFocusRangeMax = tonumber(TorqueScriptLua.getVar("$DOFPostFx::FocusRangeMax"))
    dof.maxRange = 0
    TorqueScriptLua.setVar("$DOFPostFx::FocusRangeMax", 0)
    debugging = debug
  else
    dof.maxRange = initialFocusRangeMax
    TorqueScriptLua.setVar("$DOFPostFx::FocusRangeMax", initialFocusRangeMax)
    dof.focalDist = 0
    extensions.unload(M)
    print('manual focus disabled')
  end 
end

M.onUpdate = onUpdate
M.toggleManualFocus = toggleManualFocus

return M